# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Code generated by the Google Gen AI SDK generator DO NOT EDIT.

from functools import cached_property
import io
import json
import logging
import os
from typing import Any, Optional, Union
from urllib.parse import urlencode

from . import _api_module
from . import _common
from . import _extra_utils
from . import types
from ._common import get_value_by_path as getv
from ._common import set_value_by_path as setv
from ._operations_converters import _UploadToFileSearchStoreOperation_from_mldev
from .documents import AsyncDocuments, Documents
from .pagers import AsyncPager, Pager


logger = logging.getLogger('google_genai.filesearchstores')


def _CreateFileSearchStoreConfig_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}

  if getv(from_object, ['display_name']) is not None:
    setv(parent_object, ['displayName'], getv(from_object, ['display_name']))

  return to_object


def _CreateFileSearchStoreParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['config']) is not None:
    _CreateFileSearchStoreConfig_to_mldev(
        getv(from_object, ['config']), to_object
    )

  return to_object


def _DeleteFileSearchStoreConfig_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}

  if getv(from_object, ['force']) is not None:
    setv(parent_object, ['_query', 'force'], getv(from_object, ['force']))

  return to_object


def _DeleteFileSearchStoreParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['name']) is not None:
    setv(to_object, ['_url', 'name'], getv(from_object, ['name']))

  if getv(from_object, ['config']) is not None:
    _DeleteFileSearchStoreConfig_to_mldev(
        getv(from_object, ['config']), to_object
    )

  return to_object


def _GetFileSearchStoreParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['name']) is not None:
    setv(to_object, ['_url', 'name'], getv(from_object, ['name']))

  return to_object


def _ImportFileConfig_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}

  if getv(from_object, ['custom_metadata']) is not None:
    setv(
        parent_object,
        ['customMetadata'],
        [item for item in getv(from_object, ['custom_metadata'])],
    )

  if getv(from_object, ['chunking_config']) is not None:
    setv(
        parent_object,
        ['chunkingConfig'],
        getv(from_object, ['chunking_config']),
    )

  return to_object


def _ImportFileOperation_from_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['name']) is not None:
    setv(to_object, ['name'], getv(from_object, ['name']))

  if getv(from_object, ['metadata']) is not None:
    setv(to_object, ['metadata'], getv(from_object, ['metadata']))

  if getv(from_object, ['done']) is not None:
    setv(to_object, ['done'], getv(from_object, ['done']))

  if getv(from_object, ['error']) is not None:
    setv(to_object, ['error'], getv(from_object, ['error']))

  if getv(from_object, ['response']) is not None:
    setv(
        to_object,
        ['response'],
        _ImportFileResponse_from_mldev(
            getv(from_object, ['response']), to_object
        ),
    )

  return to_object


def _ImportFileParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['file_search_store_name']) is not None:
    setv(
        to_object,
        ['_url', 'file_search_store_name'],
        getv(from_object, ['file_search_store_name']),
    )

  if getv(from_object, ['file_name']) is not None:
    setv(to_object, ['fileName'], getv(from_object, ['file_name']))

  if getv(from_object, ['config']) is not None:
    _ImportFileConfig_to_mldev(getv(from_object, ['config']), to_object)

  return to_object


def _ImportFileResponse_from_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['sdkHttpResponse']) is not None:
    setv(
        to_object, ['sdk_http_response'], getv(from_object, ['sdkHttpResponse'])
    )

  if getv(from_object, ['parent']) is not None:
    setv(to_object, ['parent'], getv(from_object, ['parent']))

  if getv(from_object, ['documentName']) is not None:
    setv(to_object, ['document_name'], getv(from_object, ['documentName']))

  return to_object


def _ListFileSearchStoresConfig_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}

  if getv(from_object, ['page_size']) is not None:
    setv(
        parent_object, ['_query', 'pageSize'], getv(from_object, ['page_size'])
    )

  if getv(from_object, ['page_token']) is not None:
    setv(
        parent_object,
        ['_query', 'pageToken'],
        getv(from_object, ['page_token']),
    )

  return to_object


def _ListFileSearchStoresParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['config']) is not None:
    _ListFileSearchStoresConfig_to_mldev(
        getv(from_object, ['config']), to_object
    )

  return to_object


def _ListFileSearchStoresResponse_from_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['sdkHttpResponse']) is not None:
    setv(
        to_object, ['sdk_http_response'], getv(from_object, ['sdkHttpResponse'])
    )

  if getv(from_object, ['nextPageToken']) is not None:
    setv(to_object, ['next_page_token'], getv(from_object, ['nextPageToken']))

  if getv(from_object, ['fileSearchStores']) is not None:
    setv(
        to_object,
        ['file_search_stores'],
        [item for item in getv(from_object, ['fileSearchStores'])],
    )

  return to_object


def _UploadToFileSearchStoreConfig_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}

  if getv(from_object, ['mime_type']) is not None:
    setv(parent_object, ['mimeType'], getv(from_object, ['mime_type']))

  if getv(from_object, ['display_name']) is not None:
    setv(parent_object, ['displayName'], getv(from_object, ['display_name']))

  if getv(from_object, ['custom_metadata']) is not None:
    setv(
        parent_object,
        ['customMetadata'],
        [item for item in getv(from_object, ['custom_metadata'])],
    )

  if getv(from_object, ['chunking_config']) is not None:
    setv(
        parent_object,
        ['chunkingConfig'],
        getv(from_object, ['chunking_config']),
    )

  return to_object


def _UploadToFileSearchStoreParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['file_search_store_name']) is not None:
    setv(
        to_object,
        ['_url', 'file_search_store_name'],
        getv(from_object, ['file_search_store_name']),
    )

  if getv(from_object, ['config']) is not None:
    _UploadToFileSearchStoreConfig_to_mldev(
        getv(from_object, ['config']), to_object
    )

  return to_object


def _UploadToFileSearchStoreResumableResponse_from_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['sdkHttpResponse']) is not None:
    setv(
        to_object, ['sdk_http_response'], getv(from_object, ['sdkHttpResponse'])
    )

  return to_object


class FileSearchStores(_api_module.BaseModule):

  @cached_property
  def documents(self) -> Documents:
    return Documents(self._api_client)

  def create(
      self, *, config: Optional[types.CreateFileSearchStoreConfigOrDict] = None
  ) -> types.FileSearchStore:
    """Creates a File Search Store.

    Args:
      config (CreateFileSearchStoreConfig | None): Optional parameters for the
        request.

    Returns:
      FileSearchStore
    """

    parameter_model = types._CreateFileSearchStoreParameters(
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _CreateFileSearchStoreParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'fileSearchStores'.format_map(request_url_dict)
      else:
        path = 'fileSearchStores'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request(
        'post', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    return_value = types.FileSearchStore._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  def get(
      self,
      *,
      name: str,
      config: Optional[types.GetFileSearchStoreConfigOrDict] = None,
  ) -> types.FileSearchStore:
    """Gets metadata about a FileSearchStore.

    Args:
      name (str): The resource name of the FileSearchStore. Example:
        `FileSearchStores/my-file-search-store-123`
      config (GetFileSearchStoreConfig | None): Optional parameters for the
        request.

    Returns:
      A FileSearchStore object containing the metadata.
    """

    parameter_model = types._GetFileSearchStoreParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _GetFileSearchStoreParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{name}'.format_map(request_url_dict)
      else:
        path = '{name}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request('get', path, request_dict, http_options)

    response_dict = {} if not response.body else json.loads(response.body)

    return_value = types.FileSearchStore._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  def delete(
      self,
      *,
      name: str,
      config: Optional[types.DeleteFileSearchStoreConfigOrDict] = None,
  ) -> None:
    """Deletes a FileSearchStore.

    Args:
      name (str): The resource name of the FileSearchStore. Example:
        `FileSearchStores/my-file-search-store-123`
      config (DeleteFileSearchStoreConfig | None): Optional parameters for the
        request.

    Returns:
      None
    """

    parameter_model = types._DeleteFileSearchStoreParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _DeleteFileSearchStoreParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{name}'.format_map(request_url_dict)
      else:
        path = '{name}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request(
        'delete', path, request_dict, http_options
    )

  def _list(
      self, *, config: Optional[types.ListFileSearchStoresConfigOrDict] = None
  ) -> types.ListFileSearchStoresResponse:
    parameter_model = types._ListFileSearchStoresParameters(
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _ListFileSearchStoresParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'fileSearchStores'.format_map(request_url_dict)
      else:
        path = 'fileSearchStores'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request('get', path, request_dict, http_options)

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _ListFileSearchStoresResponse_from_mldev(response_dict)

    return_value = types.ListFileSearchStoresResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  def _upload_to_file_search_store(
      self,
      *,
      file_search_store_name: str,
      config: Optional[types.UploadToFileSearchStoreConfigOrDict] = None,
  ) -> types.UploadToFileSearchStoreResumableResponse:
    parameter_model = types._UploadToFileSearchStoreParameters(
        file_search_store_name=file_search_store_name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _UploadToFileSearchStoreParameters_to_mldev(
          parameter_model
      )
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = (
            'upload/v1beta/{file_search_store_name}:uploadToFileSearchStore'
            .format_map(request_url_dict)
        )
      else:
        path = 'upload/v1beta/{file_search_store_name}:uploadToFileSearchStore'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request(
        'post', path, request_dict, http_options
    )

    if config is not None and getattr(
        config, 'should_return_http_response', None
    ):
      return_value = types.UploadToFileSearchStoreResumableResponse(
          sdk_http_response=response
      )
      self._api_client._verify_response(return_value)
      return return_value

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _UploadToFileSearchStoreResumableResponse_from_mldev(
          response_dict
      )

    return_value = (
        types.UploadToFileSearchStoreResumableResponse._from_response(
            response=response_dict, kwargs=parameter_model.model_dump()
        )
    )

    self._api_client._verify_response(return_value)
    return return_value

  def import_file(
      self,
      *,
      file_search_store_name: str,
      file_name: str,
      config: Optional[types.ImportFileConfigOrDict] = None,
  ) -> types.ImportFileOperation:
    """Imports a File from File Service to a FileSearchStore.

    This is a long-running operation, see aip.dev/151

    Args:
      file_search_store_name (str): The resource name of the FileSearchStore.
        Example: `fileSearchStores/my-file-search-store-123`
      file_name (str): The resource name of the File to import. Example:
        `files/abc-123`
      config (ImportFileConfig | None): Optional parameters for the request.

    Returns:
      ImportFileOperation.
    """

    parameter_model = types._ImportFileParameters(
        file_search_store_name=file_search_store_name,
        file_name=file_name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _ImportFileParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{file_search_store_name}:importFile'.format_map(
            request_url_dict
        )
      else:
        path = '{file_search_store_name}:importFile'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request(
        'post', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _ImportFileOperation_from_mldev(response_dict)

    return_value = types.ImportFileOperation._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  def upload_to_file_search_store(
      self,
      *,
      file_search_store_name: str,
      file: Union[str, os.PathLike[str], io.IOBase],
      config: Optional[types.UploadToFileSearchStoreConfigOrDict] = None,
  ) -> types.UploadToFileSearchStoreOperation:
    """Calls the API to upload a file to the given file search store.

    Args:
      file_search_store_name: The resource name of the FileSearchStore. Example:
        `fileSearchStores/file-search-store-123`
      file: A path to the file or an `IOBase` object to be uploaded. If it's an
        IOBase object, it must be opened in blocking (the default) mode and
        binary mode. In other words, do not use non-blocking mode or text mode.
        The given stream must be seekable, that is, it must be able to call
        `seek()` on 'path'.
      config: Optional parameters to set `diplay_name`, `mime_type`, and others.
    """
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    if config is None:
      user_config_dict: dict[str, Any] = {}
    elif isinstance(config, dict):
      user_config_dict = dict(config)
    else:
      user_config_dict = config.model_dump(exclude_unset=True)

    http_options, size_bytes, _ = _extra_utils.prepare_resumable_upload(
        file,
        user_http_options=user_config_dict.get('http_options'),
        user_mime_type=user_config_dict.get('mime_type'),
    )
    final_config_dict = {
        **user_config_dict,
        'http_options': http_options,
        'should_return_http_response': True,
    }
    response = self._upload_to_file_search_store(
        file_search_store_name=file_search_store_name,
        config=types.UploadToFileSearchStoreConfig(**final_config_dict),
    )
    if (
        response.sdk_http_response is None
        or response.sdk_http_response.headers is None
        or (
            'x-goog-upload-url' not in response.sdk_http_response.headers
            and 'X-Goog-Upload-URL' not in response.sdk_http_response.headers
        )
    ):
      raise KeyError(
          'Failed to upload file to file search store. Upload URL did not'
          ' returned from the upload request.'
      )
    elif 'x-goog-upload-url' in response.sdk_http_response.headers:
      upload_url = response.sdk_http_response.headers['x-goog-upload-url']
    else:
      upload_url = response.sdk_http_response.headers['X-Goog-Upload-URL']
    if isinstance(file, io.IOBase):
      upload_response = self._api_client.upload_file(
          file, upload_url, size_bytes, http_options=http_options
      )
    else:
      fs_path = os.fspath(file)
      upload_response = self._api_client.upload_file(
          fs_path, upload_url, size_bytes, http_options=http_options
      )
    response_dict = _UploadToFileSearchStoreOperation_from_mldev(
        upload_response.json
    )
    return types.UploadToFileSearchStoreOperation._from_response(
        response=response_dict, kwargs={}
    )

  def list(
      self, *, config: Optional[types.ListFileSearchStoresConfigOrDict] = None
  ) -> Pager[types.FileSearchStore]:
    """Lists FileSearchStores.

    Args:
      config (ListFileSearchStoresConfig): Optional configuration for the list
        request.

    Returns:
      A Pager object that contains one page of file search stores. When
      iterating over
      the pager, it automatically fetches the next page if there are more.

    Usage:

    .. code-block:: python
      for file_search_store in client.file_search_stores.list():
        print(f"file search store: {file_search_store.name} -
        {file_search_store.display_name}")
    """

    list_request = self._list
    return Pager(
        'file_search_stores',
        list_request,
        self._list(config=config),
        config,
    )


class AsyncFileSearchStores(_api_module.BaseModule):

  @cached_property
  def documents(self) -> AsyncDocuments:
    return AsyncDocuments(self._api_client)

  async def create(
      self, *, config: Optional[types.CreateFileSearchStoreConfigOrDict] = None
  ) -> types.FileSearchStore:
    """Creates a File Search Store.

    Args:
      config (CreateFileSearchStoreConfig | None): Optional parameters for the
        request.

    Returns:
      FileSearchStore
    """

    parameter_model = types._CreateFileSearchStoreParameters(
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _CreateFileSearchStoreParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'fileSearchStores'.format_map(request_url_dict)
      else:
        path = 'fileSearchStores'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'post', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    return_value = types.FileSearchStore._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  async def get(
      self,
      *,
      name: str,
      config: Optional[types.GetFileSearchStoreConfigOrDict] = None,
  ) -> types.FileSearchStore:
    """Gets metadata about a FileSearchStore.

    Args:
      name (str): The resource name of the FileSearchStore. Example:
        `FileSearchStores/my-file-search-store-123`
      config (GetFileSearchStoreConfig | None): Optional parameters for the
        request.

    Returns:
      A FileSearchStore object containing the metadata.
    """

    parameter_model = types._GetFileSearchStoreParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _GetFileSearchStoreParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{name}'.format_map(request_url_dict)
      else:
        path = '{name}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'get', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    return_value = types.FileSearchStore._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  async def delete(
      self,
      *,
      name: str,
      config: Optional[types.DeleteFileSearchStoreConfigOrDict] = None,
  ) -> None:
    """Deletes a FileSearchStore.

    Args:
      name (str): The resource name of the FileSearchStore. Example:
        `FileSearchStores/my-file-search-store-123`
      config (DeleteFileSearchStoreConfig | None): Optional parameters for the
        request.

    Returns:
      None
    """

    parameter_model = types._DeleteFileSearchStoreParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _DeleteFileSearchStoreParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{name}'.format_map(request_url_dict)
      else:
        path = '{name}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'delete', path, request_dict, http_options
    )

  async def _list(
      self, *, config: Optional[types.ListFileSearchStoresConfigOrDict] = None
  ) -> types.ListFileSearchStoresResponse:
    parameter_model = types._ListFileSearchStoresParameters(
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _ListFileSearchStoresParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'fileSearchStores'.format_map(request_url_dict)
      else:
        path = 'fileSearchStores'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'get', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _ListFileSearchStoresResponse_from_mldev(response_dict)

    return_value = types.ListFileSearchStoresResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  async def _upload_to_file_search_store(
      self,
      *,
      file_search_store_name: str,
      config: Optional[types.UploadToFileSearchStoreConfigOrDict] = None,
  ) -> types.UploadToFileSearchStoreResumableResponse:
    parameter_model = types._UploadToFileSearchStoreParameters(
        file_search_store_name=file_search_store_name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _UploadToFileSearchStoreParameters_to_mldev(
          parameter_model
      )
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = (
            'upload/v1beta/{file_search_store_name}:uploadToFileSearchStore'
            .format_map(request_url_dict)
        )
      else:
        path = 'upload/v1beta/{file_search_store_name}:uploadToFileSearchStore'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'post', path, request_dict, http_options
    )

    if config is not None and getattr(
        config, 'should_return_http_response', None
    ):
      return_value = types.UploadToFileSearchStoreResumableResponse(
          sdk_http_response=response
      )
      self._api_client._verify_response(return_value)
      return return_value

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _UploadToFileSearchStoreResumableResponse_from_mldev(
          response_dict
      )

    return_value = (
        types.UploadToFileSearchStoreResumableResponse._from_response(
            response=response_dict, kwargs=parameter_model.model_dump()
        )
    )

    self._api_client._verify_response(return_value)
    return return_value

  async def import_file(
      self,
      *,
      file_search_store_name: str,
      file_name: str,
      config: Optional[types.ImportFileConfigOrDict] = None,
  ) -> types.ImportFileOperation:
    """Imports a File from File Service to a FileSearchStore.

    This is a long-running operation, see aip.dev/151

    Args:
      file_search_store_name (str): The resource name of the FileSearchStore.
        Example: `fileSearchStores/my-file-search-store-123`
      file_name (str): The resource name of the File to import. Example:
        `files/abc-123`
      config (ImportFileConfig | None): Optional parameters for the request.

    Returns:
      ImportFileOperation.
    """

    parameter_model = types._ImportFileParameters(
        file_search_store_name=file_search_store_name,
        file_name=file_name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _ImportFileParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{file_search_store_name}:importFile'.format_map(
            request_url_dict
        )
      else:
        path = '{file_search_store_name}:importFile'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'post', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _ImportFileOperation_from_mldev(response_dict)

    return_value = types.ImportFileOperation._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  async def upload_to_file_search_store(
      self,
      *,
      file_search_store_name: str,
      file: Union[str, os.PathLike[str], io.IOBase],
      config: Optional[types.UploadToFileSearchStoreConfigOrDict] = None,
  ) -> types.UploadToFileSearchStoreOperation:
    """Calls the API to upload a file to the given file search store.

    Args:
      file_search_store_name: The resource name of the FileSearchStore. Example:
        `fileSearchStores/file-search-store-123`
      file: A path to the file or an `IOBase` object to be uploaded. If it's an
        IOBase object, it must be opened in blocking (the default) mode and
        binary mode. In other words, do not use non-blocking mode or text mode.
        The given stream must be seekable, that is, it must be able to call
        `seek()` on 'path'.
      config: Optional parameters to set `diplay_name`, `mime_type` and others.
    """
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    if config is None:
      user_config_dict: dict[str, Any] = {}
    elif isinstance(config, dict):
      user_config_dict = dict(config)
    else:
      user_config_dict = config.model_dump(exclude_unset=True)

    http_options, size_bytes, _ = _extra_utils.prepare_resumable_upload(
        file,
        user_http_options=user_config_dict.get('http_options'),
        user_mime_type=user_config_dict.get('mime_type'),
    )
    final_config_dict = {
        **user_config_dict,
        'http_options': http_options,
        'should_return_http_response': True,
    }
    response = await self._upload_to_file_search_store(
        file_search_store_name=file_search_store_name,
        config=types.UploadToFileSearchStoreConfig(**final_config_dict),
    )
    if (
        response.sdk_http_response is None
        or response.sdk_http_response.headers is None
        or (
            'x-goog-upload-url' not in response.sdk_http_response.headers
            and 'X-Goog-Upload-URL' not in response.sdk_http_response.headers
        )
    ):
      raise KeyError(
          'Failed to upload file to file search store. Upload URL did not'
          ' returned from the upload request.'
      )
    elif 'x-goog-upload-url' in response.sdk_http_response.headers:
      upload_url = response.sdk_http_response.headers['x-goog-upload-url']
    else:
      upload_url = response.sdk_http_response.headers['X-Goog-Upload-URL']
    if isinstance(file, io.IOBase):
      upload_response = await self._api_client.async_upload_file(
          file, upload_url, size_bytes, http_options=http_options
      )
    else:
      fs_path = os.fspath(file)
      upload_response = await self._api_client.async_upload_file(
          fs_path, upload_url, size_bytes, http_options=http_options
      )
    response_dict = _UploadToFileSearchStoreOperation_from_mldev(
        upload_response.json
    )
    return types.UploadToFileSearchStoreOperation._from_response(
        response=response_dict, kwargs={}
    )

  async def list(
      self, *, config: Optional[types.ListFileSearchStoresConfigOrDict] = None
  ) -> AsyncPager[types.FileSearchStore]:
    """Lists FileSearchStores asynchronously.

    Args:
      config (ListFileSearchStoresConfig): Optional parameters for the request,
        such as page_size.

    Returns:
      A Pager object that contains one page of FileSearchStores. When iterating
      over
      the pager, it automatically fetches the next page if there are more.

    Usage:

    .. code-block:: python
      async for file_search_store in await client.aio.file_search_stores.list():
        print(f"file search store: {file_search_store.name} -
        {file_search_store.display_name}")
    """

    list_request = self._list
    return AsyncPager(
        'file_search_stores',
        list_request,
        await self._list(config=config),
        config,
    )



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from unittest import mock
import pytest
from ... import _api_client
from ... import _extra_utils
from ... import client
from ... import models
from ... import types


TEST_NO_AFC_PART = types.Part(
    text=(
        'Okay, here is the weather in San Francisco'
        ' as of approximately  8:10 pm PST on May'
        ' 10, 2023. Please note that the weather'
        ' can change rapidly.'
    )
)

TEST_FUNCTION_CALL_PART = types.Part(
    function_call=types.FunctionCall(
        name='get_current_weather',
        args={'location': 'San Francisco'},
    )
)

TEST_FUNCTION_RESPONSE_PART = types.Part(
    function_response=types.FunctionResponse(
        name='get_current_weather',
        response={'result': 'sunny'},
    )
)

TEST_AFC_TEXT_PART = types.Part(text='San Francisco weather is sunny.')

TEST_NO_AFC_CONTENT = types.Content(
    parts=[TEST_NO_AFC_PART],
    role='model',
)

TEST_FUNCTION_CALL_CONTENT = types.Content(
    parts=[TEST_FUNCTION_CALL_PART],
    role='model',
)

TEST_FUNCTION_RESPONSE_CONTENT = types.Content(
    parts=[TEST_FUNCTION_RESPONSE_PART],
    role='user',
)

TEST_AFC_TEXT_CONTENT = types.Content(
    parts=[TEST_AFC_TEXT_PART],
    role='model',
)

TEST_AFC_HISTORY = [
    types.Content(
        parts=[types.Part(text='what is the weather in San Francisco?')],
        role='user',
    ),
    TEST_FUNCTION_CALL_CONTENT,
    TEST_FUNCTION_RESPONSE_CONTENT,
]


def get_current_weather(location: str) -> str:
  """Returns the current weather.

  Args:
    location: The location of a city and state, e.g. "San Francisco, CA".
  """
  return 'windy'


async def get_current_weather_async(location: str) -> str:
  """Returns the current weather.

  Args:
    location: The location of a city and state, e.g. "San Francisco, CA".
  """
  return 'windy'


def get_aqi_from_city(location: str) -> str:
  """Returns the aqi index of a city.

  Args:
    location: The city and State, e.g. San Francisco, CA.
  """
  return None


@pytest.fixture
def mock_api_client(vertexai=False):
  api_client = mock.MagicMock(spec=client.ApiClient)
  api_client.api_key = 'TEST_API_KEY'
  api_client._host = lambda: 'test_host'
  api_client._http_options = {'headers': {}}  # Ensure headers exist
  api_client.vertexai = vertexai
  return api_client


@pytest.fixture
def mock_get_function_response_parts_none():
  with mock.patch.object(
      _extra_utils,
      'get_function_response_parts',
  ) as mock_get_function_response_parts_none:
    mock_get_function_response_parts_none.return_value = None
    yield mock_get_function_response_parts_none


@pytest.fixture
def mock_get_function_response_parts_none_async():
  with mock.patch.object(
      _extra_utils,
      'get_function_response_parts_async',
  ) as mock_get_function_response_parts_none_async:
    mock_get_function_response_parts_none_async.return_value = None
    yield mock_get_function_response_parts_none_async


@pytest.fixture
def mock_get_function_response_parts() -> list[types.Part]:
  with mock.patch.object(
      _extra_utils, 'get_function_response_parts'
  ) as mock_get_function_response_parts:
    mock_get_function_response_parts.side_effect = [
        [TEST_FUNCTION_RESPONSE_PART],
        [],  # Breaks when the function response is not returned.
    ]
    yield mock_get_function_response_parts


@pytest.fixture
def mock_get_function_response_parts_async() -> list[types.Part]:
  with mock.patch.object(
      _extra_utils, 'get_function_response_parts_async'
  ) as mock_get_function_response_parts_async:
    mock_get_function_response_parts_async.side_effect = [
        [TEST_FUNCTION_RESPONSE_PART],
        [],  # Breaks when the function response is not returned.
    ]
    yield mock_get_function_response_parts_async


@pytest.fixture
def mock_generate_content_stream_no_afc():
  with mock.patch.object(
      models.Models, '_generate_content_stream'
  ) as mock_stream_no_afc:
    mock_stream_no_afc.return_value = [
        types.GenerateContentResponse(
            candidates=[types.Candidate(content=TEST_NO_AFC_CONTENT)]
        )
    ]
    yield mock_stream_no_afc


@pytest.fixture
def mock_generate_content_stream_with_afc():
  with mock.patch.object(
      models.Models, '_generate_content_stream'
  ) as mock_stream_with_afc:
    mock_stream_with_afc.side_effect = [
        [
            types.GenerateContentResponse(
                candidates=[types.Candidate(content=TEST_FUNCTION_CALL_CONTENT)]
            )
        ],
        [
            types.GenerateContentResponse(
                candidates=[types.Candidate(content=TEST_AFC_TEXT_CONTENT)]
            )
        ],
    ]
    yield mock_stream_with_afc


@pytest.fixture
def mock_generate_content_stream_no_afc_async():
  with mock.patch.object(
      models.AsyncModels, '_generate_content_stream'
  ) as mock_stream_no_afc:

    async def async_generator():
      yield types.GenerateContentResponse(
          candidates=[types.Candidate(content=TEST_NO_AFC_CONTENT)]
      )

    mock_stream_no_afc.return_value = async_generator()
    yield mock_stream_no_afc


@pytest.fixture
def mock_generate_content_stream_with_afc_async():
  with mock.patch.object(
      models.AsyncModels, '_generate_content_stream'
  ) as mock_stream_with_afc:

    async def async_generator_1():
      yield types.GenerateContentResponse(
          candidates=[types.Candidate(content=TEST_FUNCTION_CALL_CONTENT)]
      )

    async def async_generator_2():
      yield types.GenerateContentResponse(
          candidates=[types.Candidate(content=TEST_AFC_TEXT_CONTENT)]
      )

    mock_stream_with_afc.side_effect = [
        async_generator_1(),
        async_generator_2(),
    ]
    yield mock_stream_with_afc


def test_generate_content_stream_no_function_map(
    mock_generate_content_stream_no_afc,
    mock_get_function_response_parts_none,
):
  """Test when no function tools are provided.

  Expected to answer past weather.
  """
  models_instance = models.Models(api_client_=mock_api_client)
  stream = models_instance.generate_content_stream(
      model='test_model', contents='what is the weather in San Francisco?'
  )
  for chunk in stream:
    assert chunk.text == TEST_NO_AFC_PART.text

  assert mock_generate_content_stream_no_afc.call_count == 1
  assert mock_get_function_response_parts_none.call_count == 0


def test_generate_content_stream_afc_disabled(
    mock_generate_content_stream_with_afc,
    mock_get_function_response_parts_none,
):
  """Test when function tools are provided but AFC is disabled.

  Expected to respond with function call.
  """
  models_instance = models.Models(api_client_=mock_api_client)
  stream = models_instance.generate_content_stream(
      model='test_model',
      contents='what is the weather in San Francisco?',
      config=types.GenerateContentConfig(
          tools=[get_current_weather],
          automatic_function_calling=types.AutomaticFunctionCallingConfig(
              disable=True
          ),
      ),
  )
  for chunk in stream:
    # Work as manual function calling.
    assert chunk.candidates[0].content.parts[0].function_call

  assert mock_generate_content_stream_with_afc.call_count == 1
  assert mock_get_function_response_parts_none.call_count == 0


def test_generate_content_stream_no_function_response(
    mock_generate_content_stream_no_afc,
    mock_get_function_response_parts_none,
):
  """Test when function tools are provided and function responses are not returned.

  Expected to answer past weather.
  """
  models_instance = models.Models(api_client_=mock_api_client)
  config = types.GenerateContentConfig(tools=[get_aqi_from_city])
  stream = models_instance.generate_content_stream(
      model='test_model',
      contents='what is the weather in San Francisco?',
      config=config,
  )
  for chunk in stream:
    assert chunk.text == TEST_NO_AFC_PART.text

  assert mock_generate_content_stream_no_afc.call_count == 1
  assert mock_get_function_response_parts_none.call_count == 1


def test_generate_content_stream_with_function_tools_used(
    mock_generate_content_stream_with_afc,
    mock_get_function_response_parts,
):
  """Test when function tools are provided and function responses are returned.

  Expected to answer weather based on function response.
  """
  models_instance = models.Models(api_client_=mock_api_client)
  config = types.GenerateContentConfig(tools=[get_current_weather])
  stream = models_instance.generate_content_stream(
      model='test_model',
      contents='what is the weather in San Francisco?',
      config=config,
  )

  chunk = None
  for chunk in stream:
    assert chunk.text == TEST_AFC_TEXT_PART.text

  assert mock_generate_content_stream_with_afc.call_count == 2
  assert mock_get_function_response_parts.call_count == 2

  assert chunk is not None
  for i in range(len(chunk.automatic_function_calling_history)):
    assert chunk.automatic_function_calling_history[i].model_dump(
        exclude_none=True
    ) == TEST_AFC_HISTORY[i].model_dump(exclude_none=True)


def test_generate_content_stream_with_thought_summaries(
    mock_generate_content_stream_with_afc,
    mock_get_function_response_parts,
):
  """Test when function tools are provided and thought summaries are enabled.

  Expected to answer weather based on function response.
  """
  models_instance = models.Models(api_client_=mock_api_client)
  config = types.GenerateContentConfig(
      tools=[get_current_weather],
      thinking_config=types.ThinkingConfig(include_thoughts=True),
  )
  stream = models_instance.generate_content_stream(
      model='test_model',
      contents='what is the weather in San Francisco?',
      config=config,
  )

  chunk = None
  for chunk in stream:
    assert chunk.text == TEST_AFC_TEXT_PART.text

  assert mock_generate_content_stream_with_afc.call_count == 2
  assert mock_get_function_response_parts.call_count == 2

  assert chunk is not None
  for i in range(len(chunk.automatic_function_calling_history)):
    assert chunk.automatic_function_calling_history[i].model_dump(
        exclude_none=True
    ) == TEST_AFC_HISTORY[i].model_dump(exclude_none=True)


@pytest.mark.asyncio
async def test_generate_content_stream_no_function_map_async(
    mock_generate_content_stream_no_afc,
    mock_get_function_response_parts_none,
):
  """Test when no function tools are provided.

  Expected to answer past weather.
  """
  models_instance = models.Models(api_client_=mock_api_client)
  stream = models_instance.generate_content_stream(
      model='test_model', contents='what is the weather in San Francisco?'
  )
  for chunk in stream:
    assert chunk.text == TEST_NO_AFC_PART.text

  assert mock_generate_content_stream_no_afc.call_count == 1
  assert mock_get_function_response_parts_none.call_count == 0


@pytest.mark.asyncio
async def test_generate_content_stream_afc_disabled_async(
    mock_generate_content_stream_with_afc_async,
    mock_get_function_response_parts_none,
):
  """Test when function tools are provided but AFC is disabled.

  Expected to respond with function call.
  """
  models_instance = models.AsyncModels(api_client_=mock_api_client)
  stream = await models_instance.generate_content_stream(
      model='test_model',
      contents='what is the weather in San Francisco?',
      config=types.GenerateContentConfig(
          tools=[get_current_weather],
          automatic_function_calling=types.AutomaticFunctionCallingConfig(
              disable=True
          ),
      ),
  )
  async for chunk in stream:
    # Work as manual function calling.
    assert chunk.candidates[0].content.parts[0].function_call

  assert mock_generate_content_stream_with_afc_async.call_count == 1
  assert mock_get_function_response_parts_none.call_count == 0


@pytest.mark.asyncio
async def test_generate_content_stream_no_function_response_async(
    mock_generate_content_stream_no_afc_async,
    mock_get_function_response_parts_none_async,
):
  """Test when function tools are provided and function responses are not returned.

  Expected to answer past weather.
  """
  models_instance = models.AsyncModels(api_client_=mock_api_client)
  config = types.GenerateContentConfig(tools=[get_aqi_from_city])
  stream = await models_instance.generate_content_stream(
      model='test_model',
      contents='what is the weather in San Francisco?',
      config=config,
  )
  async for chunk in stream:
    assert chunk.text == TEST_NO_AFC_PART.text

  assert mock_generate_content_stream_no_afc_async.call_count == 1

  assert mock_get_function_response_parts_none_async.call_count == 1


@pytest.mark.asyncio
async def test_generate_content_stream_with_function_tools_used_async(
    mock_generate_content_stream_with_afc_async,
    mock_get_function_response_parts_async,
):
  """Test when function tools are provided and function responses are returned.

  Expected to answer weather based on function response.
  """
  models_instance = models.AsyncModels(api_client_=mock_api_client)
  config = types.GenerateContentConfig(tools=[get_current_weather])
  stream = await models_instance.generate_content_stream(
      model='test_model',
      contents='what is the weather in San Francisco?',
      config=config,
  )

  chunk = None
  async for chunk in stream:
    assert chunk.text == TEST_AFC_TEXT_PART.text

  assert mock_generate_content_stream_with_afc_async.call_count == 2

  assert mock_get_function_response_parts_async.call_count == 2

  assert chunk is not None
  for i in range(len(chunk.automatic_function_calling_history)):
    assert chunk.automatic_function_calling_history[i].model_dump(
        exclude_none=True
    ) == TEST_AFC_HISTORY[i].model_dump(exclude_none=True)


@pytest.mark.asyncio
async def test_generate_content_stream_with_function_async_function_used_async(
    mock_generate_content_stream_with_afc_async,
    mock_get_function_response_parts_async,
):
  """Test when function tools are provided and function responses are returned.

  Expected to answer weather based on function response.
  """
  models_instance = models.AsyncModels(api_client_=mock_api_client)
  config = types.GenerateContentConfig(tools=[get_current_weather_async])
  stream = await models_instance.generate_content_stream(
      model='test_model',
      contents='what is the weather in San Francisco?',
      config=config,
  )

  chunk = None
  async for chunk in stream:
    assert chunk.text == TEST_AFC_TEXT_PART.text

  assert mock_generate_content_stream_with_afc_async.call_count == 2

  assert mock_get_function_response_parts_async.call_count == 2

  assert chunk is not None
  for i in range(len(chunk.automatic_function_calling_history)):
    assert chunk.automatic_function_calling_history[i].model_dump(
        exclude_none=True
    ) == TEST_AFC_HISTORY[i].model_dump(exclude_none=True)


@pytest.mark.asyncio
async def test_generate_content_stream_with_thought_summaries_async(
    mock_generate_content_stream_with_afc_async,
    mock_get_function_response_parts_async,
):
  """Test when function tools are provided and thought summaries are enabled.

  Expected to answer weather based on function response.
  """
  models_instance = models.AsyncModels(api_client_=mock_api_client)
  config = types.GenerateContentConfig(
      tools=[get_current_weather],
      thinking_config=types.ThinkingConfig(include_thoughts=True),
  )
  stream = await models_instance.generate_content_stream(
      model='test_model',
      contents='what is the weather in San Francisco?',
      config=config,
  )

  chunk = None
  async for chunk in stream:
    assert chunk.text == TEST_AFC_TEXT_PART.text

  assert mock_generate_content_stream_with_afc_async.call_count == 2

  assert mock_get_function_response_parts_async.call_count == 2

  assert chunk is not None
  for i in range(len(chunk.automatic_function_calling_history)):
    assert chunk.automatic_function_calling_history[i].model_dump(
        exclude_none=True
    ) == TEST_AFC_HISTORY[i].model_dump(exclude_none=True)



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional

from .content import Content
from .._models import BaseModel

__all__ = ["Turn"]


class Turn(BaseModel):
    content: Union[str, List[Content], None] = None
    """The content of the turn."""

    role: Optional[str] = None
    """The originator of this turn.

    Must be user for input or model for
    model output.
    """



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import pytest
from ... import _transformers as t
from ... import types
from .. import pytest_helper
from . import constants

_COMPUTE_TOKENS_PARAMS = types._ComputeTokensParameters(
    model='gemini-2.5-flash',
    contents=[t.t_content('Tell me a story in 300 words.')],
)
_COMPUTE_TOKENS_PARAMS_VERTEX_CUSTOM_URL = types._ComputeTokensParameters(
    model='gemini-2.5-flash',
    contents=[t.t_content('Tell me a story in 300 words.')],
    config={'http_options': constants.VERTEX_HTTP_OPTIONS},
)
_COMPUTE_TOKENS_PARAMS_MLDEV_CUSTOM_URL = types._ComputeTokensParameters(
    model='gemini-2.5-flash',
    contents=[t.t_content('Tell me a story in 300 words.')],
    config={'http_options': constants.MLDEV_HTTP_OPTIONS},
)
_UNICODE_STRING = '这是一条unicode测试🤪❤★'

test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name='test_compute_tokens',
        exception_if_mldev='only supported in',
        parameters=types._ComputeTokensParameters(
            model='gemini-2.5-flash',
            contents=[t.t_content('Tell me a story in 300 words.')],
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_compute_tokens_vertex_custom_url',
        parameters=_COMPUTE_TOKENS_PARAMS_VERTEX_CUSTOM_URL,
        exception_if_mldev='only supported in',
    ),
    pytest_helper.TestTableItem(
        name='test_compute_tokens_mldev_custom_url',
        parameters=_COMPUTE_TOKENS_PARAMS_MLDEV_CUSTOM_URL,
        exception_if_vertex='404',
        exception_if_mldev='only supported in',
    ),
    pytest_helper.TestTableItem(
        name='test_compute_tokens_unicode',
        exception_if_mldev='only supported in',
        parameters=types._ComputeTokensParameters(
            model='gemini-2.5-flash', contents=[t.t_content(_UNICODE_STRING)]
        ),
    ),
]
pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='models.compute_tokens',
    test_table=test_table,
)


def test_token_bytes_deserialization(client):
  if client._api_client.vertexai:
    response = client.models.compute_tokens(
        model=_COMPUTE_TOKENS_PARAMS.model,
        contents=_UNICODE_STRING,
    )
    decoded_tokens = b''.join(response.tokens_info[0].tokens)
    assert (
        decoded_tokens
        == b'\xe8\xbf\x99\xe6\x98\xaf\xe4\xb8\x80\xe6\x9d\xa1unicode\xe6\xb5\x8b\xe8\xaf\x95\xf0\x9f\xa4\xaa\xe2\x9d\xa4\xe2\x98\x85'
    )
    assert decoded_tokens.decode('utf-8') == _UNICODE_STRING


@pytest.mark.asyncio
async def test_async(client):
  if client._api_client.vertexai:
    response = await client.aio.models.compute_tokens(
        model=_COMPUTE_TOKENS_PARAMS.model,
        contents=_COMPUTE_TOKENS_PARAMS.contents,
    )
    assert response
  else:
    with pytest.raises(Exception):
      await client.aio.models.compute_tokens(
          model=_COMPUTE_TOKENS_PARAMS.model,
          contents=_COMPUTE_TOKENS_PARAMS.contents,
      )


def test_different_model_names(client):
  if client._api_client.vertexai:
    response1 = client.models.compute_tokens(
        model='gemini-2.5-flash', contents=_COMPUTE_TOKENS_PARAMS.contents
    )
    assert response1
    response3 = client.models.compute_tokens(
        model='publishers/google/models/gemini-2.5-flash',
        contents=_COMPUTE_TOKENS_PARAMS.contents,
    )
    assert response3
    response4 = client.models.compute_tokens(
        model='projects/vertexsdk/locations/us-central1/publishers/google/models/gemini-2.5-flash',
        contents=_COMPUTE_TOKENS_PARAMS.contents,
    )
    assert response4



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Test files list method."""

import pytest

from ... import types
from .. import pytest_helper

test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name='test_not_empty',
        exception_if_vertex='only supported in the Gemini Developer client',
        parameters=types._ListFilesParameters(
            config=types.ListFilesConfig(
                page_size=2,
            ),
        ),
    ),
]
pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='files.list',
    test_table=test_table,
)


def test_pager(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    files = client.files.list(config={'page_size': 2})
    assert 'content-type' in files.sdk_http_response.headers
    assert files.name == 'files'
    assert files.page_size == 2
    assert len(files) <= 2

    # Iterate through all the pages. Then next_page() should raise an exception.
    for _ in files:
      pass
    with pytest.raises(IndexError, match='No more pages to fetch.'):
      files.next_page()


@pytest.mark.asyncio
async def test_async_pager(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    files = await client.aio.files.list(config={'page_size': 2})

    assert 'Content-Type' in files.sdk_http_response.headers
    assert files.name == 'files'
    assert files.page_size == 2
    assert len(files) <= 2

    # Iterate through all the pages. Then next_page() should raise an exception.
    async for _ in files:
      pass
    with pytest.raises(IndexError, match='No more pages to fetch.'):
      await files.next_page()



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for t_tools."""

import typing
from typing import Any
import pytest
from ... import _transformers as t
from ... import client as google_genai_client_module
from ... import types

_is_mcp_imported = False
if typing.TYPE_CHECKING:
  from mcp import ClientSession as McpClientSession
  from mcp import types as mcp_types

  _is_mcp_imported = True
else:
  try:
    from mcp import ClientSession as McpClientSession
    from mcp import types as mcp_types

    _is_mcp_imported = True
  except ImportError:
    _is_mcp_imported = False


@pytest.fixture
def client(use_vertex):
  if use_vertex:
    yield google_genai_client_module.Client(
        vertexai=use_vertex, project='test-project', location='test-location'
    )
  else:
    yield google_genai_client_module.Client(
        vertexai=use_vertex, api_key='test-api-key'
    )


@pytest.mark.usefixtures('client')
def test_empty(client):
  assert t.t_tools(client, []) == []


@pytest.mark.usefixtures('client')
def test_function(client):
  def test_func(arg1: str, arg2: int):
    pass

  assert t.t_tools(client, [test_func]) == [
      types.Tool(
          function_declarations=[
              types.FunctionDeclaration(
                  name='test_func',
                  parameters=types.Schema(
                      type='OBJECT',
                      properties={
                          'arg1': types.Schema(type='STRING'),
                          'arg2': types.Schema(type='INTEGER'),
                      },
                      required=['arg1', 'arg2'],
                  ),
              )
          ]
      )
  ]


@pytest.mark.usefixtures('client')
def test_tool(client):
  tool = types.Tool(
      function_declarations=[
          types.FunctionDeclaration(
              name='tool',
              description='tool-description',
              parameters=types.Schema(
                  type='OBJECT',
                  properties={
                      'key1': types.Schema(type='STRING'),
                      'key2': types.Schema(type='NUMBER'),
                  },
              ),
          )
      ]
  )
  assert t.t_tools(client, [tool]) == [tool]


@pytest.mark.usefixtures('client')
def test_mcp_tool(client):
  if not _is_mcp_imported:
    return

  mcp_tool = mcp_types.Tool(
      name='tool',
      description='tool-description',
      inputSchema={
          'type': 'object',
          'properties': {
              'key1': {
                  'type': 'string',
              },
              'key2': {
                  'type': 'number',
              },
          },
      },
  )
  assert t.t_tools(client, [mcp_tool]) == [
      types.Tool(
          function_declarations=[
              types.FunctionDeclaration(
                  name='tool',
                  description='tool-description',
                  parameters=types.Schema(
                      type='OBJECT',
                      properties={
                          'key1': types.Schema(type='STRING'),
                          'key2': types.Schema(type='NUMBER'),
                      },
                  ),
              )
          ]
      )
  ]


@pytest.mark.usefixtures('client')
def test_multiple_tools(client):
  if not _is_mcp_imported:
    return

  mcp_tool1 = mcp_types.Tool(
      name='tool1',
      description='tool1-description',
      inputSchema={
          'type': 'object',
          'properties': {
              'key1': {
                  'type': 'string',
              },
          },
      },
  )
  mcp_tool2 = mcp_types.Tool(
      name='tool2',
      description='tool2-description',
      inputSchema={
          'type': 'object',
          'properties': {
              'key1': {
                  'type': 'number',
              },
          },
      },
  )
  assert t.t_tools(client, [mcp_tool1, mcp_tool2]) == [
      types.Tool(
          function_declarations=[
              types.FunctionDeclaration(
                  name='tool1',
                  description='tool1-description',
                  parameters=types.Schema(
                      type='OBJECT',
                      properties={
                          'key1': types.Schema(type='STRING'),
                      },
                  ),
              ),
              types.FunctionDeclaration(
                  name='tool2',
                  description='tool2-description',
                  parameters=types.Schema(
                      type='OBJECT',
                      properties={
                          'key1': types.Schema(type='NUMBER'),
                      },
                  ),
              ),
          ]
      ),
  ]



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for models.generate_content_stream()."""

from .... import types as genai_types
from ... import pytest_helper
from pydantic import BaseModel


class _GenerateContentStreamParameters(BaseModel):
    model: str
    contents: str


def generate_content_stream(client, parameters):
  response = client.models.generate_content_stream(
      model=parameters.model,
      contents=parameters.contents,
  )
  for _ in response:
    pass


test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name='test_generate_content_stream',
        parameters=_GenerateContentStreamParameters(
            model='gemini-2.5-flash',
            contents='The quick brown fox jumps over the lazy dog.',
        ),
    ),
]

pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='generate_content_stream',
    test_table=test_table,
)

pytest_plugins = ('pytest_asyncio',)



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests tools in the _common module."""

from enum import Enum
import inspect
import logging
import textwrap
import typing
from typing import List, Optional
import warnings

import pydantic
import pytest

from ... import _common
from ... import types
from ... import errors


def test_warn_once():
  @_common.experimental_warning('Warning!')
  def func():
    pass

  with warnings.catch_warnings(record=True) as w:
    func()
    func()

  assert len(w) == 1
  assert w[0].category == errors.ExperimentalWarning

def test_warn_at_call_line():
  @_common.experimental_warning('Warning!')
  def func():
    pass

  with warnings.catch_warnings(record=True) as captured_warnings:
    call_line = inspect.currentframe().f_lineno + 1
    func()

  assert captured_warnings[0].lineno == call_line


def test_is_struct_type():
  assert _common._is_struct_type(list[dict[str, typing.Any]])
  assert _common._is_struct_type(typing.List[typing.Dict[str, typing.Any]])
  assert not _common._is_struct_type(list[dict[str, int]])
  assert not _common._is_struct_type(list[dict[int, typing.Any]])
  assert not _common._is_struct_type(list[str])
  assert not _common._is_struct_type(dict[str, typing.Any])
  assert not _common._is_struct_type(typing.List[typing.Dict[str, int]])
  assert not _common._is_struct_type(typing.List[typing.Dict[int, typing.Any]])
  assert not _common._is_struct_type(typing.List[str])
  assert not _common._is_struct_type(typing.Dict[str, typing.Any])



@pytest.mark.parametrize(
    "test_id, initial_target, update_dict, expected_target",
    [
        (
            "simple_update",
            {"a": 1, "b": 2},
            {"b": 3, "c": 4},
            {"a": 1, "b": 3, "c": 4},
        ),
        (
            "nested_update",
            {"a": 1, "b": {"x": 10, "y": 20}},
            {"b": {"y": 30, "z": 40}, "c": 3},
            {"a": 1, "b": {"x": 10, "y": 30, "z": 40}, "c": 3},
        ),
        (
            "add_new_nested_dict",
            {"a": 1},
            {"b": {"x": 10, "y": 20}},
            {"a": 1, "b": {"x": 10, "y": 20}},
        ),
        (
            "empty_target",
            {},
            {"a": 1, "b": {"x": 10}},
            {"a": 1, "b": {"x": 10}},
        ),
        (
            "empty_update",
            {"a": 1, "b": {"x": 10}},
            {},
            {"a": 1, "b": {"x": 10}},
        ),
        (
            "overwrite_non_dict_with_dict",
            {"a": 1, "b": 2},
            {"b": {"x": 10}},
            {"a": 1, "b": {"x": 10}},
        ),
        (
            "overwrite_dict_with_non_dict",
            {"a": 1, "b": {"x": 10}},
            {"b": 2},
            {"a": 1, "b": 2},
        ),
        (
            "deeper_nesting",
            {"a": {"b": {"c": 1, "d": 2}, "e": 3}},
            {"a": {"b": {"d": 4, "f": 5}, "g": 6}, "h": 7},
            {"a": {"b": {"c": 1, "d": 4, "f": 5}, "e": 3, "g": 6}, "h": 7},
        ),
        (
            "different_value_types",
            {"key1": "string_val", "key2": {"nested_int": 100}},
            {"key1": 123, "key2": {"nested_list": [1, 2, 3]}, "key3": True},
            {
                "key1": 123,
                "key2": {"nested_int": 100, "nested_list": [1, 2, 3]},
                "key3": True,
            },
        ),
        (
            "update_with_empty_nested_dict", # Existing nested dict in target should not be cleared
            {"a": {"b": 1}},
            {"a": {}},
            {"a": {"b": 1}},
        ),
        (
            "target_with_empty_nested_dict",
            {"a": {}},
            {"a": {"b": 1}},
            {"a": {"b": 1}},
        ),
        (
            "key_case_alignment_check",
            {"first_name": "John", "contact_info": {"email_address": "john@example.com"}},
            {"firstName": "Jane", "contact_info": {"email_address": "jane@example.com", "phone_number": "123"}},
            {"first_name": "Jane", "contact_info": {"email_address": "jane@example.com", "phone_number": "123"}},
        )
    ],
)
def test_recursive_dict_update(
    test_id: str, initial_target: dict, update_dict: dict, expected_target: dict
):
  _common.recursive_dict_update(initial_target, update_dict)
  assert initial_target == expected_target


@pytest.mark.parametrize(
    "test_id, initial_target, update_dict, expected_target, expect_warning, expected_log_message_part",
    [
        (
            "type_match_int",
            {"a": 1},
            {"a": 2},
            {"a": 2},
            False,
            "",
        ),
        (
            "type_match_dict",
            {"a": {"b": 1}},
            {"a": {"b": 2}},
            {"a": {"b": 2}},
            False,
            "",
        ),
        (
            "type_mismatch_int_to_str",
            {"a": 1},
            {"a": "hello"},
            {"a": "hello"},
            True,
            "Type mismatch for key 'a'. Existing type: <class 'int'>, new type: <class 'str'>. Overwriting.",
        ),
        (
            "type_mismatch_dict_to_int",
            {"a": {"b": 1}},
            {"a": 100},
            {"a": 100},
            True,
            "Type mismatch for key 'a'. Existing type: <class 'dict'>, new type: <class 'int'>. Overwriting.",
        ),
        (
            "type_mismatch_int_to_dict",
            {"a": 100},
            {"a": {"b": 1}},
            {"a": {"b": 1}},
            True,
            "Type mismatch for key 'a'. Existing type: <class 'int'>, new type: <class 'dict'>. Overwriting.",
        ),
        ("add_new_key", {"a": 1}, {"b": "new"}, {"a": 1, "b": "new"}, False, ""),
    ],
)
def test_recursive_dict_update_type_warnings(test_id, initial_target, update_dict, expected_target, expect_warning, expected_log_message_part, caplog):
    _common.recursive_dict_update(initial_target, update_dict)
    assert initial_target == expected_target
    if expect_warning:
        assert len(caplog.records) == 1
        assert caplog.records[0].levelname == "WARNING"
        assert expected_log_message_part in caplog.records[0].message
    else:
        for record in caplog.records:
            if record.levelname == "WARNING" and expected_log_message_part in record.message:
                 pytest.fail(f"Unexpected warning logged for {test_id}: {record.message}")


@pytest.mark.parametrize(
    "test_id, target_dict, update_dict, expected_aligned_dict",
    [
        (
            "simple_snake_to_camel",
            {"first_name": "John", "last_name": "Doe"},
            {"firstName": "Jane", "lastName": "Doe"},
            {"first_name": "Jane", "last_name": "Doe"},
        ),
        (
            "simple_camel_to_snake",
            {"firstName": "John", "lastName": "Doe"},
            {"first_name": "Jane", "last_name": "Doe"},
            {"firstName": "Jane", "lastName": "Doe"},
        ),
        (
            "nested_dict_alignment",
            {"user_info": {"contact_details": {"email_address": ""}}},
            {"userInfo": {"contactDetails": {"emailAddress": "test@example.com"}}},
            {"user_info": {"contact_details": {"email_address": "test@example.com"}}},
        ),
        (
            "list_of_dicts_alignment",
            {"users_list": [{"user_id": 0, "user_name": ""}]},
            {"usersList": [{"userId": 1, "userName": "Alice"}]},
            {"users_list": [{"userId": 1, "userName": "Alice"}]},
        ),
        (
            "list_of_dicts_alignment_mixed_case_in_update",
            {"users_list": [{"user_id": 0, "user_name": ""}]},
            {"usersList": [{"user_id": 1, "UserName": "Alice"}]},
            {"users_list": [{"user_id": 1, "UserName": "Alice"}]},
        ),
        (
            "list_of_dicts_different_lengths_update_longer",
            {"items_data": [{"item_id": 0}]},
            {"itemsData": [{"itemId": 1}, {"item_id": 2, "itemName": "Extra"}]},
            {"items_data": [{"itemId": 1}, {"item_id": 2, "itemName": "Extra"}]},
        ),
        (
            "list_of_dicts_different_lengths_target_longer",
            {"items_data": [{"item_id": 0, "item_name": ""}, {"item_id": 1}]},
            {"itemsData": [{"itemId": 10}]},
            {"items_data": [{"itemId": 10}]},
        ),
        (
            "no_matching_keys_preserves_update_case",
            {"key_one": 1},
            {"KEY_TWO": 2, "keyThree": 3},
            {"KEY_TWO": 2, "keyThree": 3},
        ),
        (
            "mixed_match_and_no_match",
            {"first_name": "John", "age_years": 30},
            {"firstName": "Jane", "AGE_YEARS": 28, "occupation_title": "Engineer"},
            {"first_name": "Jane", "age_years": 28, "occupation_title": "Engineer"},
        ),
        (
            "empty_target_dict",
            {},
            {"new_key": "new_value", "anotherKey": "anotherValue"},
            {"new_key": "new_value", "anotherKey": "anotherValue"},
        ),
        (
            "empty_update_dict",
            {"existing_key": "value"},
            {},
            {},
        ),
        (
            "target_has_non_dict_value_for_nested_key",
            {"config_settings": 123},
            {"configSettings": {"themeName": "dark"}},
            {"config_settings": {"themeName": "dark"}}, # Overwrites as per recursive_dict_update logic
        ),
        (
            "update_has_non_dict_value_for_nested_key",
            {"config_settings": {"theme_name": "light"}},
            {"configSettings": "dark_theme_string"},
            {"config_settings": "dark_theme_string"}, # Overwrites
        ),
         (
            "deeply_nested_with_lists",
            {"level_one": {"list_items": [{"item_name": "", "item_value": 0}]}},
            {"levelOne": {"listItems": [{"itemName": "Test", "itemValue": 100}, {"itemName": "Test2", "itemValue": 200}]}},
            {"level_one": {"list_items": [{"itemName": "Test", "itemValue": 100}, {"itemName": "Test2", "itemValue": 200}]}},
        ),
    ],
)
def test_align_key_case(
    test_id: str, target_dict: dict, update_dict: dict, expected_aligned_dict: dict
):
  aligned_dict = _common.align_key_case(target_dict, update_dict)
  assert aligned_dict == expected_aligned_dict, f"Test failed for: {test_id}"



class SimpleModel(_common.BaseModel):
  name: str
  value: int
  is_active: bool = True
  none_field: Optional[str] = None


class Chain(_common.BaseModel):
  id: int
  child: Optional["Chain"] = None


Chain.model_rebuild()


class Tree(_common.BaseModel):
  id: int
  children: List["Tree"] = pydantic.Field(default_factory=list)


Tree.model_rebuild()


class ReprFalseModel(_common.BaseModel):
  visible: str
  hidden: str = pydantic.Field("secret", repr=False)


class NonPydantic:

  def __repr__(self):
    return "NonPydantic(\n  attr='value'\n)"


class MyEnum(Enum):
  ONE = 1
  TWO = 2


class EmptyModel(_common.BaseModel):
  pass


def test_repr_simple_model_defaults_and_no_none():
  obj = SimpleModel(name="Test Name", value=123)
  expected = textwrap.dedent("""
    SimpleModel(
      is_active=True,
      name='Test Name',
      value=123
    )
    """).strip()
  assert repr(obj) == expected


def test_repr_empty_model():
  obj = EmptyModel()
  expected = "EmptyModel()"
  assert repr(obj) == expected


def test_repr_nested_model():
  obj = Chain(id=1, child=Chain(id=2))
  expected = textwrap.dedent("""
    Chain(
      child=Chain(
        id=2
      ),
      id=1
    )
    """).strip()
  assert repr(obj) == expected


def test_repr_circular_model():
  obj1 = Chain(id=1)
  obj2 = Chain(id=2)
  obj1.child = obj2
  obj2.child = obj1  # Circular reference
  expected = textwrap.dedent("""
    Chain(
      child=Chain(
        child=<... Circular reference ...>,
        id=2
      ),
      id=1
    )
    """).strip()

  assert repr(obj1) == expected


def test_repr_circular_list():
  my_list = [1, 2]
  my_list.append(my_list)
  expected = textwrap.dedent("""
    [
      1,
      2,
      <... Circular reference ...>,
    ]
    """).strip()
  assert _common._pretty_repr(my_list) == expected


def test_repr_circular_dict():
  my_dict = {"a": 1}
  my_dict["self"] = my_dict
  expected = textwrap.dedent("""
    {
      'a': 1,
      'self': <... Circular reference ...>
    }
    """).strip()
  assert _common._pretty_repr(my_dict) == expected


def test_repr_max_items():
  lst = list(range(10))
  dct = {i: i for i in range(10)}
  st = set(range(10))
  tpl = tuple(range(10))

  assert (
      "<... 5 more items ...>" in
      _common._pretty_repr(lst, max_items=5)
  )
  assert (
      "<dict len=10>" in _common._pretty_repr(dct, max_items=5))
  assert (
      "<... 5 more items ...>" in _common._pretty_repr(st, max_items=5)
  )
  assert (
      "<... 5 more items ...>" in _common._pretty_repr(tpl, max_items=5)
  )


def test_repr_max_len_bytes():
  b_data = b"a" * 100
  assert len(_common._pretty_repr(b_data, max_len=90)) == 90 + 3
  assert repr(b_data) == _common._pretty_repr(b_data, max_len=200)


def test_repr_max_depth_dict():
  nested = {'a': {'a': {'a': {'a': 'a', 'b': 'b'}}}}
  assert "{<... 2 items at Max depth ...>}" in _common._pretty_repr(nested, depth=3)


def test_repr_max_depth_list():
  nested = [[[["d", "e", "e", "p"]]]]
  assert "[<... 4 items at Max depth ...>]" in _common._pretty_repr(nested, depth=3)


def test_repr_collections():
  obj = {
      "set": {3, 1, 2},
      "tuple": (4, 5, 6),
      "dict": {"b": 2, "a": 1},
      "list": [7, 8, 9],
  }
  expected = textwrap.dedent("""
    {
      'dict': {
        'a': 1,
        'b': 2
      },
      'list': [
        7,
        8,
        9,
      ],
      'set': {
        1,
        2,
        3,
      },
      'tuple': (
        4,
        5,
        6,
      )
    }
    """).strip()
  assert _common._pretty_repr(obj) == expected


def test_tuple_collections():
  obj = {
      "tuple0": (),
      "tuple1": (1,),
      "tuple2": (1, 2),
  }
  expected = textwrap.dedent("""
    {
      'tuple0': (),
      'tuple1': (
        1,
      ),
      'tuple2': (
        1,
        2,
      )
    }
    """).strip()
  assert _common._pretty_repr(obj) == expected


def test_repr_empty_collections():
  assert _common._pretty_repr([]) == "[]"
  assert _common._pretty_repr({}) == "{}"
  assert (
      _common._pretty_repr(set()) == "set()"
  )
  assert _common._pretty_repr(tuple()) == "()"
  assert (
      _common._pretty_repr({"empty_set": set()}) ==
      textwrap.dedent("""
        {
          'empty_set': set()
        }
      """).strip()
  )


def test_repr_strings():
  s1 = "line one"
  exp1 = "'line one'"
  assert _common._pretty_repr(s1) == exp1

  s2 = 'line one\nline two with """ inside'
  exp2 = '"""line one\nline two with \\"\\"\\" inside"""'
  assert _common._pretty_repr(s2) == exp2

  s3 = 'A string with """ inside'
  exp3 = '\'A string with """ inside\''
  assert _common._pretty_repr(s3) == exp3


def test_repr_repr_false():
  obj = ReprFalseModel(visible="show", hidden="hide")
  result = repr(obj)
  assert "visible='show'" in result
  assert "hidden" not in result
  expected = textwrap.dedent("""
    ReprFalseModel(
      visible='show'
    )
    """).strip()
  assert result == expected


def test_repr_none_fields():
  obj = SimpleModel(name="Only Name", value=0, none_field=None)
  result = repr(obj)
  assert "none_field" not in result
  expected = textwrap.dedent("""
    SimpleModel(
      is_active=True,
      name='Only Name',
      value=0
    )
    """).strip()
  assert result == expected


def test_repr_other_types():
  np = NonPydantic()
  en = MyEnum.TWO
  obj = {"np": np, "en": en}
  expected = textwrap.dedent("""
    {
      'en': <MyEnum.TWO: 2>,
      'np': NonPydantic(
          attr='value'
        )
    }
    """).strip()
  assert _common._pretty_repr(obj) == expected


def test_repr_indent_delta():
  obj = SimpleModel(name="Indent Test", value=1)
  expected = textwrap.dedent("""
    SimpleModel(
        is_active=True,
        name='Indent Test',
        value=1
    )
    """).strip()
  assert _common._pretty_repr(obj, indent_delta=4) == expected


def test_repr_complex_object():
  obj = types.GenerateContentResponse(
      automatic_function_calling_history=[],
      candidates=[
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(
                          text="""There isn't a single "best" LLM, as the ideal choice highly depends on your specific needs, use case, budget, and priorities. The field is evolving incredibly fast, with new models and improvements being released constantly.

However, we can talk about the **leading contenders** and what they are generally known for:..."""
                      )
                  ],
                  role="model"
              ),
              finish_reason=types.FinishReason.STOP,
              index=0
          )
      ],
      model_version='models/gemini-2.5-flash-preview-05-20',
      usage_metadata=types.GenerateContentResponseUsageMetadata(
          candidates_token_count=1086,
          prompt_token_count=7,
          prompt_tokens_details=[
              types.ModalityTokenCount(
                  modality=types.MediaModality.TEXT,
                  token_count=7
              )
          ],
          thoughts_token_count=860,
          total_token_count=1953
      )
  )

  expected = textwrap.dedent("""
      GenerateContentResponse(
        automatic_function_calling_history=[],
        candidates=[
          Candidate(
            content=Content(
              parts=[
                Part(
                  text=\"\"\"There isn't a single "best" LLM, as the ideal choice highly depends on your specific needs, use case, budget, and priorities. The field is evolving incredibly fast, with new models and improvements being released constantly.

      However, we can talk about the **leading contenders** and what they are generally known for:...\"\"\"
                ),
              ],
              role='model'
            ),
            finish_reason=<FinishReason.STOP: 'STOP'>,
            index=0
          ),
        ],
        model_version='models/gemini-2.5-flash-preview-05-20',
        usage_metadata=GenerateContentResponseUsageMetadata(
          candidates_token_count=1086,
          prompt_token_count=7,
          prompt_tokens_details=[
            ModalityTokenCount(
              modality=<MediaModality.TEXT: 'TEXT'>,
              token_count=7
            ),
          ],
          thoughts_token_count=860,
          total_token_count=1953
        )
      )
  """).strip()
  assert repr(obj) == expected


def test_move_value_by_path():
  """Test move_value_by_path function with array wildcard notation."""
  data = {
      "requests": [
          {
              "request": {
                  "content": {
                      "parts": [
                          {
                              "text": "1"
                          }
                      ]
                  }
              },
              "outputDimensionality": 64
          },
          {
              "request": {
                  "content": {
                      "parts": [
                          {
                              "text": "2"
                          }
                      ]
                  }
              },
              "outputDimensionality": 64
          },
          {
              "request": {
                  "content": {
                      "parts": [
                          {
                              "text": "3"
                          }
                      ]
                  }
              },
              "outputDimensionality": 64
          }
      ]
  }

  paths = {'requests[].*': 'requests[].request.*'}
  _common.move_value_by_path(data, paths)

  expected = {
      "requests": [
          {
              "request": {
                  "content": {
                      "parts": [
                          {
                              "text": "1"
                          }
                      ]
                  },
                  "outputDimensionality": 64
              }
          },
          {
              "request": {
                  "content": {
                      "parts": [
                          {
                              "text": "2"
                          }
                      ]
                  },
                  "outputDimensionality": 64
              }
          },
          {
              "request": {
                  "content": {
                      "parts": [
                          {
                              "text": "3"
                          }
                      ]
                  },
                  "outputDimensionality": 64
              }
          }
      ]
  }

  assert data == expected


def test_check_field_type_mismatches_no_warning_for_correct_types(caplog):
  """Test that no warning is logged when types match."""

  class ModelA(_common.BaseModel):
    value: int

  class TestModel(_common.BaseModel):
    model_a: ModelA

  # Should not warn - dict will be converted to ModelA by Pydantic
  data = {"model_a": {"value": 123}}

  with caplog.at_level(logging.WARNING, logger="google_genai._common"):
    result = TestModel.model_validate(data)

  assert result.model_a.value == 123
  assert len(caplog.records) == 0


def test_check_field_type_mismatches_warns_on_pydantic_type_mismatch(caplog):
  """Test that warning is logged when Pydantic model types mismatch."""

  class ModelA(_common.BaseModel):
    value: int

  class ModelB(_common.BaseModel):
    value: str

  class TestModel(_common.BaseModel):
    model_field: ModelA

  # Create an instance of ModelB (wrong type)
  model_b_instance = ModelB(value="test")

  # Pass the wrong Pydantic model instance
  data = {"model_field": model_b_instance}

  with caplog.at_level(logging.WARNING, logger="google_genai._common"):
    TestModel._check_field_type_mismatches(data)

  assert len(caplog.records) == 1
  assert "Type mismatch in TestModel.model_field" in caplog.records[0].message
  assert "expected ModelA, got ModelB" in caplog.records[0].message


def test_check_field_type_mismatches_no_warning_for_none_values(caplog):
  """Test that no warning is logged for None values."""

  class ModelA(_common.BaseModel):
    value: int

  class TestModel(_common.BaseModel):
    model_field: Optional[ModelA] = None

  data = {"model_field": None}

  with caplog.at_level(logging.WARNING, logger="google_genai._common"):
    result = TestModel.model_validate(data)

  assert result.model_field is None
  assert len(caplog.records) == 0


def test_check_field_type_mismatches_no_warning_for_missing_fields(caplog):
  """Test that no warning is logged for missing fields."""

  class ModelA(_common.BaseModel):
    value: int

  class TestModel(_common.BaseModel):
    model_field: Optional[ModelA] = None

  data = {}

  with caplog.at_level(logging.WARNING, logger="google_genai._common"):
    result = TestModel.model_validate(data)

  assert result.model_field is None
  assert len(caplog.records) == 0


def test_check_field_type_mismatches_no_warning_for_primitive_types(caplog):
  """Test that no warning is logged for primitive type mismatches."""

  class TestModel(_common.BaseModel):
    int_field: int
    str_field: str

  # Even though we're passing wrong primitive types, we should not warn
  # (Pydantic will handle validation)
  data = {"int_field": "123", "str_field": "test"}

  with caplog.at_level(logging.WARNING, logger="google_genai._common"):
    # This will succeed because Pydantic can coerce "123" to int
    result = TestModel.model_validate(data)

  assert result.int_field == 123
  assert result.str_field == "test"
  assert len(caplog.records) == 0


def test_check_field_type_mismatches_handles_optional_unwrapping(caplog):
  """Test that Optional types are properly unwrapped before checking."""

  class ModelA(_common.BaseModel):
    value: int

  class ModelB(_common.BaseModel):
    value: str

  class TestModel(_common.BaseModel):
    model_field: Optional[ModelA] = None

  # Pass wrong Pydantic model type
  model_b_instance = ModelB(value="test")
  data = {"model_field": model_b_instance}

  with caplog.at_level(logging.WARNING, logger="google_genai._common"):
    TestModel._check_field_type_mismatches(data)

  assert len(caplog.records) == 1
  assert "expected ModelA, got ModelB" in caplog.records[0].message


def test_check_field_type_mismatches_no_warning_for_correct_pydantic_instance(caplog):
  """Test that no warning is logged when correct Pydantic instance is provided."""

  class ModelA(_common.BaseModel):
    value: int

  class TestModel(_common.BaseModel):
    model_field: ModelA

  # Pass correct Pydantic model instance
  model_a_instance = ModelA(value=42)
  data = {"model_field": model_a_instance}

  with caplog.at_level(logging.WARNING, logger="google_genai._common"):
    result = TestModel.model_validate(data)

  assert result.model_field.value == 42
  assert len(caplog.records) == 0


def test_check_field_type_mismatches_with_multiple_fields(caplog):
  """Test checking multiple fields with mixed scenarios."""

  class ModelA(_common.BaseModel):
    value: int

  class ModelB(_common.BaseModel):
    value: str

  class TestModel(_common.BaseModel):
    field_a: ModelA
    field_b: Optional[ModelA] = None
    field_c: str

  model_b_instance = ModelB(value="wrong")
  data = {
      "field_a": model_b_instance,  # Wrong type - should warn
      "field_b": None,  # None - should not warn
      "field_c": "test",  # Primitive - should not warn
  }

  with caplog.at_level(logging.WARNING, logger="google_genai._common"):
    TestModel._check_field_type_mismatches(data)

  # Should only warn about field_a
  assert len(caplog.records) == 1
  assert "field_a" in caplog.records[0].message
  assert "expected ModelA, got ModelB" in caplog.records[0].message


def test_check_field_type_mismatches_generic_type_no_error(caplog):
  """Test that validation doesn't crash on generic types like list[str]."""
  class TestModel(_common.BaseModel):
    tags: list[str]

  data = {"tags": ["a", "b"]}

  with caplog.at_level(logging.WARNING, logger="google_genai._common"):
    TestModel.model_validate(data)

  assert len(caplog.records) == 0



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Constants for caches tests."""

CACHED_CONTENT_NAME_MLDEV = 'cachedContents/o239k1gxzz0juy9wqstndhncr85krehehf551hqh'
CACHED_CONTENT_NAME_VERTEX = 'cachedContents/2164089915711684608'

VERTEX_HTTP_OPTIONS = {
    'api_version': 'v1beta1',
    'base_url': 'https://us-central1-aiplatform.googleapis.com/',
}
MLDEV_HTTP_OPTIONS = {
    'api_version': 'v1beta',
    'base_url': 'https://generativelanguage.googleapis.com/',
}


# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Code generated by the Google Gen AI SDK generator DO NOT EDIT.

import io
import json
import logging
import os
from typing import Any, Optional, Union
from urllib.parse import urlencode

import google.auth

from . import _api_client
from . import _api_module
from . import _common
from . import _extra_utils
from . import _transformers as t
from . import types
from ._common import get_value_by_path as getv
from ._common import set_value_by_path as setv
from .pagers import AsyncPager, Pager


logger = logging.getLogger('google_genai.files')


def _CreateFileParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['file']) is not None:
    setv(to_object, ['file'], getv(from_object, ['file']))

  return to_object


def _CreateFileResponse_from_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['sdkHttpResponse']) is not None:
    setv(
        to_object, ['sdk_http_response'], getv(from_object, ['sdkHttpResponse'])
    )

  return to_object


def _DeleteFileParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['name']) is not None:
    setv(
        to_object, ['_url', 'file'], t.t_file_name(getv(from_object, ['name']))
    )

  return to_object


def _DeleteFileResponse_from_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['sdkHttpResponse']) is not None:
    setv(
        to_object, ['sdk_http_response'], getv(from_object, ['sdkHttpResponse'])
    )

  return to_object


def _GetFileParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['name']) is not None:
    setv(
        to_object, ['_url', 'file'], t.t_file_name(getv(from_object, ['name']))
    )

  return to_object


def _InternalRegisterFilesParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['uris']) is not None:
    setv(to_object, ['uris'], getv(from_object, ['uris']))

  return to_object


def _ListFilesConfig_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}

  if getv(from_object, ['page_size']) is not None:
    setv(
        parent_object, ['_query', 'pageSize'], getv(from_object, ['page_size'])
    )

  if getv(from_object, ['page_token']) is not None:
    setv(
        parent_object,
        ['_query', 'pageToken'],
        getv(from_object, ['page_token']),
    )

  return to_object


def _ListFilesParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['config']) is not None:
    _ListFilesConfig_to_mldev(getv(from_object, ['config']), to_object)

  return to_object


def _ListFilesResponse_from_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['sdkHttpResponse']) is not None:
    setv(
        to_object, ['sdk_http_response'], getv(from_object, ['sdkHttpResponse'])
    )

  if getv(from_object, ['nextPageToken']) is not None:
    setv(to_object, ['next_page_token'], getv(from_object, ['nextPageToken']))

  if getv(from_object, ['files']) is not None:
    setv(to_object, ['files'], [item for item in getv(from_object, ['files'])])

  return to_object


def _RegisterFilesResponse_from_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['sdkHttpResponse']) is not None:
    setv(
        to_object, ['sdk_http_response'], getv(from_object, ['sdkHttpResponse'])
    )

  if getv(from_object, ['files']) is not None:
    setv(to_object, ['files'], [item for item in getv(from_object, ['files'])])

  return to_object


class Files(_api_module.BaseModule):

  def _list(
      self, *, config: Optional[types.ListFilesConfigOrDict] = None
  ) -> types.ListFilesResponse:
    parameter_model = types._ListFilesParameters(
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _ListFilesParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'files'.format_map(request_url_dict)
      else:
        path = 'files'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request('get', path, request_dict, http_options)

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _ListFilesResponse_from_mldev(response_dict)

    return_value = types.ListFilesResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )
    return_value.sdk_http_response = types.HttpResponse(
        headers=response.headers
    )
    self._api_client._verify_response(return_value)
    return return_value

  def _create(
      self,
      *,
      file: types.FileOrDict,
      config: Optional[types.CreateFileConfigOrDict] = None,
  ) -> types.CreateFileResponse:
    parameter_model = types._CreateFileParameters(
        file=file,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _CreateFileParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'upload/v1beta/files'.format_map(request_url_dict)
      else:
        path = 'upload/v1beta/files'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request(
        'post', path, request_dict, http_options
    )

    if config is not None and getattr(
        config, 'should_return_http_response', None
    ):
      return_value = types.CreateFileResponse(sdk_http_response=response)
      self._api_client._verify_response(return_value)
      return return_value

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _CreateFileResponse_from_mldev(response_dict)

    return_value = types.CreateFileResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  def get(
      self, *, name: str, config: Optional[types.GetFileConfigOrDict] = None
  ) -> types.File:
    """Retrieves the file information from the service.

    Args:
      name (str): The name identifier for the file to retrieve.
      config (GetFileConfig): Optional, configuration for the get method.

    Returns:
      File: The file information.

    Usage:

    .. code-block:: python

      file = client.files.get(name='files/...')
      print(file.uri)
    """

    parameter_model = types._GetFileParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _GetFileParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'files/{file}'.format_map(request_url_dict)
      else:
        path = 'files/{file}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request('get', path, request_dict, http_options)

    response_dict = {} if not response.body else json.loads(response.body)

    return_value = types.File._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  def delete(
      self, *, name: str, config: Optional[types.DeleteFileConfigOrDict] = None
  ) -> types.DeleteFileResponse:
    """Deletes a remotely stored file.

    Args:
      name (str): The name identifier for the file to delete.
      config (DeleteFileConfig): Optional, configuration for the delete method.

    Returns:
      DeleteFileResponse: The response for the delete method

    Usage:

    .. code-block:: python

      client.files.delete(name='files/...')
    """

    parameter_model = types._DeleteFileParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _DeleteFileParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'files/{file}'.format_map(request_url_dict)
      else:
        path = 'files/{file}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request(
        'delete', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _DeleteFileResponse_from_mldev(response_dict)

    return_value = types.DeleteFileResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )
    return_value.sdk_http_response = types.HttpResponse(
        headers=response.headers
    )
    self._api_client._verify_response(return_value)
    return return_value

  def _register_files(
      self,
      *,
      uris: list[str],
      config: Optional[types.RegisterFilesConfigOrDict] = None,
  ) -> types.RegisterFilesResponse:
    parameter_model = types._InternalRegisterFilesParameters(
        uris=uris,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _InternalRegisterFilesParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'files:register'.format_map(request_url_dict)
      else:
        path = 'files:register'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request(
        'post', path, request_dict, http_options
    )

    if config is not None and getattr(
        config, 'should_return_http_response', None
    ):
      return_value = types.RegisterFilesResponse(sdk_http_response=response)
      self._api_client._verify_response(return_value)
      return return_value

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _RegisterFilesResponse_from_mldev(response_dict)

    return_value = types.RegisterFilesResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  def upload(
      self,
      *,
      file: Union[str, os.PathLike[str], io.IOBase],
      config: Optional[types.UploadFileConfigOrDict] = None,
  ) -> types.File:
    """Calls the API to upload a file using a supported file service.

    Args:
      file: A path to the file or an `IOBase` object to be uploaded. If it's an
        IOBase object, it must be opened in blocking (the default) mode and
        binary mode. In other words, do not use non-blocking mode or text mode.
        The given stream must be seekable, that is, it must be able to call
        `seek()` on 'path'.
      config: Optional parameters to set `diplay_name`, `mime_type`, and `name`.
    """
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    config_model = types.UploadFileConfig()
    if config:
      if isinstance(config, dict):
        config_model = types.UploadFileConfig(**config)
      else:
        config_model = config
      file_obj = types.File(
          mime_type=config_model.mime_type,
          name=config_model.name,
          display_name=config_model.display_name,
      )
    else:  # if not config
      file_obj = types.File()
    if file_obj.name is not None and not file_obj.name.startswith('files/'):
      file_obj.name = f'files/{file_obj.name}'

    http_options, size_bytes, mime_type = _extra_utils.prepare_resumable_upload(
        file,
        user_http_options=config_model.http_options,
        user_mime_type=config_model.mime_type,
    )
    file_obj.size_bytes = size_bytes
    file_obj.mime_type = mime_type
    response = self._create(
        file=file_obj,
        config=types.CreateFileConfig(
            http_options=http_options, should_return_http_response=True
        ),
    )

    if (
        response.sdk_http_response is None
        or response.sdk_http_response.headers is None
        or 'x-goog-upload-url' not in response.sdk_http_response.headers
    ):
      raise KeyError(
          'Failed to create file. Upload URL did not returned from the create'
          ' file request.'
      )
    upload_url = response.sdk_http_response.headers['x-goog-upload-url']

    if isinstance(file, io.IOBase):
      return_file = self._api_client.upload_file(
          file, upload_url, file_obj.size_bytes, http_options=http_options
      )
    else:
      fs_path = os.fspath(file)
      return_file = self._api_client.upload_file(
          fs_path, upload_url, file_obj.size_bytes, http_options=http_options
      )

    return types.File._from_response(
        response=return_file.json['file'],
        kwargs=config_model.model_dump() if config else {},
    )

  def download(
      self,
      *,
      file: Union[str, types.File, types.Video, types.GeneratedVideo],
      config: Optional[types.DownloadFileConfigOrDict] = None,
  ) -> bytes:
    """Downloads a file's data from storage.

    Files created by `upload` can't be downloaded. You can tell which files are
    downloadable by checking the `source` or `download_uri` property.

    Note: This method returns the data as bytes. For `Video` and
    `GeneratedVideo` objects there is an additional side effect, that it also
    sets the `video_bytes` property on the `Video` object.

    Args:
      file (str): A file name, uri, or file object. Identifying which file to
        download.
      config (DownloadFileConfigOrDict): Optional, configuration for the get
        method.

    Returns:
      File: The file data as bytes.

    Usage:

    .. code-block:: python

      for file client.files.list():
        if file.download_uri is not None:
          break
      else:
        raise ValueError('No files found with a `download_uri`.')
      data = client.files.download(file=file)
      # data = client.files.download(file=file.name)
      # data = client.files.download(file=file.download_uri)

      video = types.Video(uri=file.uri)
      video_bytes = client.files.download(file=video)
      video.video_bytes
    """
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )

    config_model = None
    if config:
      if isinstance(config, dict):
        config_model = types.DownloadFileConfig(**config)
      else:
        config_model = config

    if isinstance(file, types.File) and file.download_uri is None:
      raise ValueError(
          "Only generated files can be downloaded, uploaded files can't be "
          'downloaded. You can tell which files are downloadable by checking '
          'the `source` or `download_uri` property.'
      )
    name = t.t_file_name(file)

    path = f'files/{name}:download'

    query_params = {'alt': 'media'}
    path = f'{path}?{urlencode(query_params)}'
    http_options = None
    if getv(config_model, ['http_options']) is not None:
      http_options = getv(config_model, ['http_options'])

    data = self._api_client.download_file(
        path,
        http_options=http_options,
    )

    if isinstance(file, types.Video):
      file.video_bytes = data
    elif isinstance(file, types.GeneratedVideo) and file.video is not None:
      file.video.video_bytes = data

    return data

  def register_files(
      self,
      *,
      auth: google.auth.credentials.Credentials,
      uris: list[str],
      config: Optional[types.RegisterFilesConfigOrDict] = None,
  ) -> types.RegisterFilesResponse:
    """Registers gcs files with the file service."""
    if not isinstance(auth, google.auth.credentials.Credentials):
      raise ValueError(
          'auth must be a google.auth.credentials.Credentials object.'
      )
    if config is None:
      config = types.RegisterFilesConfig()
    else:
      config = types.RegisterFilesConfig.model_validate(config)
      config = config.model_copy(deep=True)

    http_options = config.http_options or types.HttpOptions()
    headers = http_options.headers or {}
    headers = {k.lower(): v for k, v in headers.items()}

    token = _api_client.get_token_from_credentials(self._api_client, auth)
    headers['authorization'] = f'Bearer {token}'

    if auth.quota_project_id:
      headers['x-goog-user-project'] = auth.quota_project_id

    http_options.headers = headers
    config.http_options = http_options

    return self._register_files(uris=uris, config=config)

  def list(
      self, *, config: Optional[types.ListFilesConfigOrDict] = None
  ) -> Pager[types.File]:
    """Lists all files from the service.

    Args:
      config (ListFilesConfig): Optional, configuration for the list method.

    Returns:
      A Pager object that contains one page of files. When iterating over
      the pager, it automatically fetches the next page if there are more.

    Usage:

    .. code-block:: python

      for file in client.files.list(config={'page_size': 10}):
        print(file.name)
    """

    list_request = self._list
    return Pager(
        'files',
        list_request,
        self._list(config=config),
        config,
    )


class AsyncFiles(_api_module.BaseModule):

  async def _list(
      self, *, config: Optional[types.ListFilesConfigOrDict] = None
  ) -> types.ListFilesResponse:
    parameter_model = types._ListFilesParameters(
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _ListFilesParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'files'.format_map(request_url_dict)
      else:
        path = 'files'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'get', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _ListFilesResponse_from_mldev(response_dict)

    return_value = types.ListFilesResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )
    return_value.sdk_http_response = types.HttpResponse(
        headers=response.headers
    )
    self._api_client._verify_response(return_value)
    return return_value

  async def _create(
      self,
      *,
      file: types.FileOrDict,
      config: Optional[types.CreateFileConfigOrDict] = None,
  ) -> types.CreateFileResponse:
    parameter_model = types._CreateFileParameters(
        file=file,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _CreateFileParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'upload/v1beta/files'.format_map(request_url_dict)
      else:
        path = 'upload/v1beta/files'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'post', path, request_dict, http_options
    )

    if config is not None and getattr(
        config, 'should_return_http_response', None
    ):
      return_value = types.CreateFileResponse(sdk_http_response=response)
      self._api_client._verify_response(return_value)
      return return_value

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _CreateFileResponse_from_mldev(response_dict)

    return_value = types.CreateFileResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  async def get(
      self, *, name: str, config: Optional[types.GetFileConfigOrDict] = None
  ) -> types.File:
    """Retrieves the file information from the service.

    Args:
      name (str): The name identifier for the file to retrieve.
      config (GetFileConfig): Optional, configuration for the get method.

    Returns:
      File: The file information.

    Usage:

    .. code-block:: python

      file = await client.aio.files.get(name='files/...')
      print(file.uri)
    """

    parameter_model = types._GetFileParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _GetFileParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'files/{file}'.format_map(request_url_dict)
      else:
        path = 'files/{file}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'get', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    return_value = types.File._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  async def delete(
      self, *, name: str, config: Optional[types.DeleteFileConfigOrDict] = None
  ) -> types.DeleteFileResponse:
    """Deletes a remotely stored file.

    Args:
      name (str): The name identifier for the file to delete.
      config (DeleteFileConfig): Optional, configuration for the delete method.

    Returns:
      DeleteFileResponse: The response for the delete method

    Usage:

    .. code-block:: python

      await client.aio.files.delete(name='files/...')
    """

    parameter_model = types._DeleteFileParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _DeleteFileParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'files/{file}'.format_map(request_url_dict)
      else:
        path = 'files/{file}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'delete', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _DeleteFileResponse_from_mldev(response_dict)

    return_value = types.DeleteFileResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )
    return_value.sdk_http_response = types.HttpResponse(
        headers=response.headers
    )
    self._api_client._verify_response(return_value)
    return return_value

  async def _register_files(
      self,
      *,
      uris: list[str],
      config: Optional[types.RegisterFilesConfigOrDict] = None,
  ) -> types.RegisterFilesResponse:
    parameter_model = types._InternalRegisterFilesParameters(
        uris=uris,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _InternalRegisterFilesParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'files:register'.format_map(request_url_dict)
      else:
        path = 'files:register'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'post', path, request_dict, http_options
    )

    if config is not None and getattr(
        config, 'should_return_http_response', None
    ):
      return_value = types.RegisterFilesResponse(sdk_http_response=response)
      self._api_client._verify_response(return_value)
      return return_value

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _RegisterFilesResponse_from_mldev(response_dict)

    return_value = types.RegisterFilesResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  async def upload(
      self,
      *,
      file: Union[str, os.PathLike[str], io.IOBase],
      config: Optional[types.UploadFileConfigOrDict] = None,
  ) -> types.File:
    """Calls the API to upload a file asynchronously using a supported file service.

    Args:
      file: A path to the file or an `IOBase` object to be uploaded. If it's an
        IOBase object, it must be opened in blocking (the default) mode and
        binary mode. In other words, do not use non-blocking mode or text mode.
        The given stream must be seekable, that is, it must be able to call
        `seek()` on 'path'.
      config: Optional parameters to set `diplay_name`, `mime_type`, and `name`.
    """
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    config_model = types.UploadFileConfig()
    if config:
      if isinstance(config, dict):
        config_model = types.UploadFileConfig(**config)
      else:
        config_model = config
      file_obj = types.File(
          mime_type=config_model.mime_type,
          name=config_model.name,
          display_name=config_model.display_name,
      )
    else:  # if not config
      file_obj = types.File()
    if file_obj.name is not None and not file_obj.name.startswith('files/'):
      file_obj.name = f'files/{file_obj.name}'

    http_options, size_bytes, mime_type = _extra_utils.prepare_resumable_upload(
        file,
        user_http_options=config_model.http_options,
        user_mime_type=config_model.mime_type,
    )
    file_obj.size_bytes = size_bytes
    file_obj.mime_type = mime_type
    response = await self._create(
        file=file_obj,
        config=types.CreateFileConfig(
            http_options=http_options, should_return_http_response=True
        ),
    )
    if (
        response.sdk_http_response is None
        or response.sdk_http_response.headers is None
        or (
            'x-goog-upload-url' not in response.sdk_http_response.headers
            and 'X-Goog-Upload-URL' not in response.sdk_http_response.headers
        )
    ):
      raise KeyError(
          'Failed to create file. Upload URL did not returned from the create'
          ' file request.'
      )
    elif 'x-goog-upload-url' in response.sdk_http_response.headers:
      upload_url = response.sdk_http_response.headers['x-goog-upload-url']
    else:
      upload_url = response.sdk_http_response.headers['X-Goog-Upload-URL']

    if isinstance(file, io.IOBase):
      return_file = await self._api_client.async_upload_file(
          file, upload_url, file_obj.size_bytes, http_options=http_options
      )
    else:
      fs_path = os.fspath(file)
      return_file = await self._api_client.async_upload_file(
          fs_path, upload_url, file_obj.size_bytes, http_options=http_options
      )

    return types.File._from_response(
        response=return_file.json['file'],
        kwargs=config_model.model_dump() if config else {},
    )

  async def download(
      self,
      *,
      file: Union[str, types.File],
      config: Optional[types.DownloadFileConfigOrDict] = None,
  ) -> bytes:
    """Downloads a file's data from the file service.

    The Vertex-AI implementation of the API foes not include the file service.

    Files created by `upload` can't be downloaded. You can tell which files are
    downloadable by checking the `download_uri` property.

    Args:
      File (str): A file name, uri, or file object. Identifying which file to
        download.
      config (DownloadFileConfigOrDict): Optional, configuration for the get
        method.

    Returns:
      File: The file data as bytes.

    Usage:

    .. code-block:: python

      for file client.files.list():
        if file.download_uri is not None:
          break
      else:
        raise ValueError('No files found with a `download_uri`.')
      data = client.files.download(file=file)
      # data = client.files.download(file=file.name)
      # data = client.files.download(file=file.uri)
    """
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )

    config_model = None
    if config:
      if isinstance(config, dict):
        config_model = types.DownloadFileConfig(**config)
      else:
        config_model = config

    name = t.t_file_name(file)

    path = f'files/{name}:download'

    http_options = None
    if getv(config_model, ['http_options']) is not None:
      http_options = getv(config_model, ['http_options'])

    query_params = {'alt': 'media'}
    if query_params:
      path = f'{path}?{urlencode(query_params)}'

    data = await self._api_client.async_download_file(
        path,
        http_options=http_options,
    )

    return data

  async def register_files(
      self,
      *,
      auth: google.auth.credentials.Credentials,
      uris: list[str],
      config: Optional[types.RegisterFilesConfigOrDict] = None,
  ) -> types.RegisterFilesResponse:
    """Registers gcs files with the file service."""
    if not isinstance(auth, google.auth.credentials.Credentials):
      raise ValueError(
          'auth must be a google.auth.credentials.Credentials object.'
      )
    if config is None:
      config = types.RegisterFilesConfig()
    else:
      config = types.RegisterFilesConfig.model_validate(config)
      config = config.model_copy(deep=True)

    http_options = config.http_options or types.HttpOptions()
    headers = http_options.headers or {}
    headers = {k.lower(): v for k, v in headers.items()}

    token = await _api_client.async_get_token_from_credentials(
        self._api_client, auth
    )
    headers['authorization'] = f'Bearer {token}'

    if auth.quota_project_id:
      headers['x-goog-user-project'] = auth.quota_project_id

    http_options.headers = headers
    config.http_options = http_options

    return await self._register_files(uris=uris, config=config)

  async def list(
      self, *, config: Optional[types.ListFilesConfigOrDict] = None
  ) -> AsyncPager[types.File]:
    """Lists all files from the service asynchronously.

    Args:
      config (ListFilesConfig): Optional, configuration for the list method.

    Returns:
      A Pager object that contains one page of files. When iterating over
      the pager, it automatically fetches the next page if there are more.

    Usage:

    .. code-block:: python

      async for file in await client.aio.files.list(config={'page_size': 10}):
        print(file.name)
    """

    list_request = self._list
    return AsyncPager(
        'files',
        list_request,
        await self._list(config=config),
        config,
    )
